package cn.com.infosec.crypto.generators;

import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.DerivationFunction;
import cn.com.infosec.crypto.DerivationParameters;
import cn.com.infosec.crypto.Digest;
import cn.com.infosec.crypto.params.MGFParameters;

public class MGF1BytesGenerator implements DerivationFunction {
   private Digest digest;
   private byte[] seed;
   private int hLen;

   public MGF1BytesGenerator(Digest digest) {
      this.digest = digest;
      this.hLen = digest.getDigestSize();
   }

   private void ItoOSP(int i, byte[] sp) {
      sp[0] = (byte)(i >>> 24);
      sp[1] = (byte)(i >>> 16);
      sp[2] = (byte)(i >>> 8);
      sp[3] = (byte)(i >>> 0);
   }

   public void init(DerivationParameters param) {
      if (param instanceof MGFParameters) {
         this.seed = ((MGFParameters)param).getSeed();
      } else {
         throw new IllegalArgumentException("MGF parameters required for MGF1Generator");
      }
   }

   public Digest getDigest() {
      return this.digest;
   }

   public int generateBytes(byte[] out, int outOff, int len) {
      if (out.length - len < outOff) {
         throw new DataLengthException("output buffer too small");
      } else {
         byte[] var4 = new byte[this.hLen];
         byte var5 = 4;
         byte[] var6 = new byte[4];
         int var7 = 0;
         this.digest.reset();
         if (len > this.hLen) {
            do {
               this.ItoOSP(var7, var6);
               byte[] var10007 = this.seed;
               this.digest.update(var10007, 0, var10007.length);
               this.digest.update(var6, 0, var5);
               this.digest.doFinal(var4, 0);
               int var8;
               int var9 = outOff + var7 * (var8 = this.hLen);
               System.arraycopy(var4, 0, out, var9, var8);
            } while(++var7 < len / this.hLen);
         }

         if (var7 * this.hLen < len) {
            this.ItoOSP(var7, var6);
            byte[] var10009 = this.seed;
            this.digest.update(var10009, 0, var10009.length);
            this.digest.update(var6, 0, var5);
            this.digest.doFinal(var4, 0);
            int var10;
            outOff += var7 * (var10 = this.hLen);
            var10 = len - var7 * var10;
            System.arraycopy(var4, 0, out, outOff, var10);
         }

         return len;
      }
   }
}
